@@ -16,8 +16,6 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -703,7 +701,7 @@ PHP_FUNCTION(ssh2_auth_pubkey_file)
 }
 /* }}} */
 
-/* {{{ proto bool ssh2_auth_hostbased_file(resource session, string username, string local_hostname, string pubkeyfile, string privkeyfile[, string passphrase[, string local_username]])
+/* {{{ proto bool ssh2_auth_hostbased_file(resource session, string username, string hostname, string pubkeyfile, string privkeyfile[, string passphrase[, string local_username]])
  * Authenticate using a hostkey
  */
 PHP_FUNCTION(ssh2_auth_hostbased_file)
@@ -772,9 +770,8 @@ PHP_FUNCTION(ssh2_forward_listen)
 
 	data = emalloc(sizeof(php_ssh2_listener_data));
 	data->session = session;
-	data->session_rsrcid = Z_LVAL_P(zsession);
-	//TODO Sean-Der
-	//zend_list_addref(data->session_rsrcid);
+	data->session_rsrc = Z_RES_P(zsession);
+	Z_ADDREF_P(zsession);
 	data->listener = listener;
 
 	RETURN_RES(zend_register_resource(data, le_ssh2_listener));
@@ -810,7 +807,7 @@ PHP_FUNCTION(ssh2_forward_accept)
 	channel_data->channel = channel;
 	channel_data->streamid = 0;
 	channel_data->is_blocking = 0;
-	channel_data->session_rsrcid = data->session_rsrcid;
+	channel_data->session_rsrc = data->session_rsrc;
 	channel_data->refcount = NULL;
 
 	stream = php_stream_alloc(&php_ssh2_channel_stream_ops, channel_data, 0, "r+");
@@ -820,8 +817,12 @@ PHP_FUNCTION(ssh2_forward_accept)
 		libssh2_channel_free(channel);
 		RETURN_FALSE;
 	}
-	//TODO Sean-Der
-	//zend_list_addref(channel_data->session_rsrcid);
+
+#if PHP_VERSION_ID < 70300
+	GC_REFCOUNT(channel_data->session_rsrc)++;
+#else
+	GC_ADDREF(channel_data->session_rsrc);
+#endif
 
 	php_stream_to_zval(stream, return_value);
 }
@@ -975,9 +976,8 @@ PHP_FUNCTION(ssh2_publickey_init)
 
 	data = emalloc(sizeof(php_ssh2_pkey_subsys_data));
 	data->session = session;
-	data->session_rsrcid = Z_RES_P(zsession)->handle;
-	//TODO Sean-Der
-	//zend_list_addref(data->session_rsrcid);
+	data->session_rsrc = Z_RES_P(zsession);
+	Z_ADDREF_P(zsession);
 	data->pkey = pkey;
 
 	RETURN_RES(zend_register_resource(data, le_ssh2_pkey_subsys));
@@ -1280,8 +1280,7 @@ static void php_ssh2_listener_dtor(zend_resource *rsrc)
 	LIBSSH2_LISTENER *listener = data->listener;
 
 	libssh2_channel_forward_cancel(listener);
-	// TODO Sean-Der
-	//zend_list_delete(data->session_rsrcid);
+	zend_list_delete(data->session_rsrc);
 	efree(data);
 }
 
@@ -1291,8 +1290,7 @@ static void php_ssh2_pkey_subsys_dtor(zend_resource *rsrc)
 	LIBSSH2_PUBLICKEY *pkey = data->pkey;
 
 	libssh2_publickey_shutdown(pkey);
-	// TODO Sean-Der
-	//zend_list_delete(data->session_rsrcid);
+	zend_list_delete(data->session_rsrc);
 	efree(data);
 }
 
@@ -1369,21 +1367,23 @@ PHP_MINFO_FUNCTION(ssh2)
 
 /* {{{ ZEND_BEGIN_ARG_INFO
  */
-ZEND_BEGIN_ARG_INFO(arginfo_ssh2_connect, 2)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ssh2_connect, 0, 0, 1)
  	ZEND_ARG_INFO(0, host)
  	ZEND_ARG_INFO(0, port)
+ 	ZEND_ARG_INFO(0, methods)
+ 	ZEND_ARG_INFO(0, callbacks)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO(arginfo_ssh2_disconnect, 1)
- 	ZEND_ARG_INFO(0, resource)
+ 	ZEND_ARG_INFO(0, session)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO(arginfo_ssh2_methods_negotiated, 1)
- 	ZEND_ARG_INFO(0, resource)
+ 	ZEND_ARG_INFO(0, session)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_ssh2_fingerprint, 0, 0, 2)
- 	ZEND_ARG_INFO(0, resource)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ssh2_fingerprint, 0, 0, 1)
+ 	ZEND_ARG_INFO(0, session)
  	ZEND_ARG_INFO(0, flags)
 ZEND_END_ARG_INFO()
 
@@ -1449,14 +1449,14 @@ ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_ssh2_scp_send, 0, 0, 3)
  	ZEND_ARG_INFO(0, session)
- 	ZEND_ARG_INFO(0, remote_file)
  	ZEND_ARG_INFO(0, local_file)
+ 	ZEND_ARG_INFO(0, remote_file)
  	ZEND_ARG_INFO(0, create_mode)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO(arginfo_ssh2_fetch_stream, 2)
  	ZEND_ARG_INFO(0, channel)
- 	ZEND_ARG_INFO(0, stream_id)
+ 	ZEND_ARG_INFO(0, streamid)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO(arginfo_ssh2_sftp, 1)
